package com.bwie.xxxxxx.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Case;
import com.bwie.common.domain.CaseRequest;
import com.bwie.common.domain.User;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.xxxxxx.mapper.CaseMapper;
import com.bwie.xxxxxx.service.CaseService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author Handsome boy
 * @Package com.bwie.xxxxxx.service.impl
 * @date 2023/12/8 20:21
 */
@Service
public class CaseServiceImpl implements CaseService {

    final CaseMapper caseMapper;
    final HttpServletRequest request;
    final RedisTemplate<String, String> redisTemplate;

    public CaseServiceImpl(CaseMapper caseMapper, HttpServletRequest request, RedisTemplate<String, String> redisTemplate) {
        this.caseMapper = caseMapper;
        this.request = request;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 添加病例
     *
     * @param cases;
     */
    @Override
    public Result<Object> insertCase(Case cases) {
        int row;
        try {
            row = caseMapper.insertCase(cases);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (row == 0) {
            throw new IllegalAccessError("添加异常");
        }
        return Result.success("添加病例成功");
    }

    /**
     * 删除病例
     *
     * @param caseId;
     */
    @Override
    public Result<Object> deleteCase(Integer caseId) {
        int row;
        try {
            row = caseMapper.deleteCase(caseId);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (row == 0) {
            throw new IllegalAccessError("删除异常");
        }
        return Result.success("删除病例成功");
    }

    /**
     * 修改病例
     *
     * @param cases;
     */
    @Override
    public Result<Object> updateCase(Case cases) {
        int row;
        try {
            row = caseMapper.updateCase(cases);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (row == 0) {
            throw new IllegalAccessError("修改异常");
        }
        return Result.success("修改病例成功");
    }

    @Override
    public Result<PageResult<Case>> selCase(CaseRequest caseRequest) {
        // 调用PageHelper的startPage方法，传入当前页码和每页条数
        PageHelper.startPage(caseRequest.getPageNum(), caseRequest.getPageSize());
        //获取登录人信息
        User user = getToken();
        if (user == null){
            return Result.error("未登录");
        }
        caseRequest.setRoleId(user.getRole());
//         调用caseMapper的selCase方法，传入CaseRequest参数
        List<Case> list = caseMapper.selCase(caseRequest);
        // 调用PageInfo的构造方法，传入list参数
        PageInfo<Case> page = new PageInfo<>(list);
        // 调用Result的success方法，传入PageResult.toPageResult方法，传入page.getTotal()和list参数
        return PageResult.toResult(page.getTotal() ,list);
    }


    /**
     * 获取登录人信息
     *
     * @return ;
     */
    private User getToken() {
        //获取token
        String token = request.getHeader("token");
        //获取用户key
        String userKey = JwtUtils.getUserKey(token);
        //从redis中获取用户信息
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        //将用户信息转换为User对象
        return JSON.parseObject(s, User.class);
    }


}
